/*
 * Tuxcall -- Call waiting detector for Linux
 * Copyright (C) 2001-2002 Angel Luis Jimenez Martinez
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <gtk/gtk.h>

#include "common.h"
#include "xcommon.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "aux.h"
#include "xaux.h"
#include "conf.h"
#include "xconf.h"

#define MAX_N_VALUES 185
#define LINE_WIDTH 1

extern enum tAction Action;
extern int End, FramesUpdate;
extern float Freq;
extern char HangCmd[];
extern GtkWidget * mainWin;
extern GtkWidget * callWin;
extern void (*FuncOnCall)();
extern char Device[], HangCmd[];

static GdkPixmap * pixmap = NULL;
static GdkGC * gc;
static GdkColor colNormal, colTone, colText;
static GdkFont * font;
GtkWidget * aboutWin;
GtkWidget * confWin;

/*
void updateGraph( float value, float limit )
{
    GtkWidget * w;
    GdkColor color;
    GdkGC * gc;
    static int frame= 1;

    if (value < limit)
    {
        color.red= 0;
        color.green= 0;
        color.blue= 65530;
    }
    else
    {
        color.red= 65530;
        color.green= 0;
        color.blue= 0;
    }


        w= lookup_widget (GTK_WIDGET(mainWin), "drawArea");
        gc= gdk_gc_new(w->window);
        gdk_color_alloc(gtk_widget_get_colormap(w), &color);

        gdk_gc_set_foreground(gc, &color);
        gdk_draw_rectangle(pixmap, w->style->black_gc, TRUE
            , w->allocation.width-LINE_WIDTH, 0 , LINE_WIDTH
            , w->allocation.height );
        gdk_draw_rectangle(pixmap, gc, TRUE, w->allocation.width-LINE_WIDTH
            , w->allocation.height - value*10  
            , LINE_WIDTH, value*10 );
        gdk_draw_pixmap(pixmap, w->style->fg_gc[GTK_WIDGET_STATE (w)], pixmap
            , LINE_WIDTH, 0, 0, 0, w->allocation.width-LINE_WIDTH
            , w->allocation.height);
        


    if ( frame == FramesUpdate )
    {
        frame= 1;
        gtk_widget_draw(GTK_WIDGET(w), NULL);
    }         
    else
        frame++;
}
*/

void showCallWin()
{
    gdk_beep();
    gtk_widget_show( callWin );
    gtk_grab_add( callWin );
}

void defineGc()
{
    GtkWidget * w;

    colTone.red= 65535;
    colTone.green= 0;
    colTone.blue= 0;

    colNormal.red= 0;
    colNormal.green= 0;
    colNormal.blue= 65535;
    
    colText.red= 65535;
    colText.green= 65535;
    colText.blue= 65535;

    w= lookup_widget (GTK_WIDGET(mainWin), "drawArea");
    /*gc= gdk_gc_new(w->window);*/
    gc= gdk_gc_new(w->window);
    gdk_color_alloc(gtk_widget_get_colormap(w), &colTone);
    gdk_color_alloc(gtk_widget_get_colormap(w), &colNormal);
    gdk_color_alloc(gtk_widget_get_colormap(w), &colText);

    font= gdk_font_load("fixed");
}

/*
 * Updates sound graph
 * Receives last value and sound limit (everything above it is a call tone)
 */
void updateGraph( float value, float limit )
{
    GtkWidget * w;
    static int nValues= 0, frame= 1;
    static float values[MAX_N_VALUES], maxValue= 0;
    static float scale;
    int i;
    char msg[50];

    if ( nValues <  MAX_N_VALUES )
        nValues++;
    else    
    {
        memmove( values, &values[1], (MAX_N_VALUES-1)*sizeof(float) );
        nValues= MAX_N_VALUES;
    }    

    if ( value > maxValue )
        maxValue= value;
    else    
        maxValue*= 0.995;
        
    values[nValues - 1]= value;
    

    if ( frame >= FramesUpdate )
    {
        frame= 1;
        
        w= lookup_widget (GTK_WIDGET(mainWin), "drawArea");
    
        gdk_draw_rectangle(pixmap, w->style->black_gc, TRUE, 0, 0
            , w->allocation.width, w->allocation.height);  
    
        scale= (float)w->allocation.height / maxValue;
        /*printf( "scale= %f\n", scale );*/

        gdk_gc_set_foreground(gc, &colNormal);

        for( i=0; i<nValues; i++ )    
        {
            if ( values[i] > limit )
            {
                gdk_gc_set_foreground(gc, &colTone);
                gdk_draw_rectangle(pixmap, gc, TRUE
                    , i*LINE_WIDTH
                    , w->allocation.height - values[i]*scale
                    , LINE_WIDTH
                    , values[i]*scale );
                gdk_gc_set_foreground(gc, &colNormal);
            }
            else
                gdk_draw_rectangle(pixmap, gc, TRUE
                    , i*LINE_WIDTH
                    , w->allocation.height - values[i]*scale
                    , LINE_WIDTH
                    , values[i]*scale );
        }        

        gdk_gc_set_foreground(gc, &colText);
        snprintf( msg, 49, "%.1f", maxValue );
        gdk_draw_string( pixmap, font, gc, 5, 10, msg );
    
        if (w->window == NULL)
            return;

        gdk_draw_pixmap(w->window, w->style->fg_gc[GTK_WIDGET_STATE (w)], pixmap
            , 0, 0, 0, 0, w->allocation.width, w->allocation.height);
    }            
    else
        frame++;
        
}

gboolean
on_drawArea_configure_event          (GtkWidget       *widget,
                                        GdkEventConfigure *event,
                                        gpointer         user_data)
{
    if (pixmap)
        gdk_pixmap_unref(pixmap);

    pixmap = gdk_pixmap_new(widget->window, widget->allocation.width,
        widget->allocation.height, -1);
    gdk_draw_rectangle (pixmap, widget->style->black_gc, TRUE, 0, 0,
        widget->allocation.width, widget->allocation.height);

#ifdef DEBUG
    printf("configure\n");    
#endif    

    return TRUE;
}


gboolean
on_drawArea_expose_event             (GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         user_data)
{
    if (widget->window == NULL)
        return FALSE;

    gdk_draw_pixmap(widget->window,
        widget->style->fg_gc[GTK_WIDGET_STATE (widget)], pixmap,
        event->area.x, event->area.y, event->area.x, event->area.y,
        event->area.width, event->area.height);

#ifdef DEBUG
    printf("expose\n");  
#endif    

    return FALSE;
}


/*
 * For closing application
 */
gboolean
the_end                                (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
    FILE * file;
    char confFile[FNAMELEN], msg[200];

    End= 1;  

    snprintf( confFile, FNAMELEN-1, "%s/.%s", getenv("HOME"), CONF );
    file= fopen( confFile, "w" );
    if ( file == NULL )
    {
        snprintf( msg, 200-1, _("Can't save to \nconfig file \"%s\"")
            , confFile );
        quickMsg( msg );
    }
    else
        xSaveConf( file, Device, HangCmd, Freq, FramesUpdate, Action );

    fclose( file );

    gtk_main_quit();

    return FALSE;
}


void
on_about_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    char str[300];
    GtkLabel * textAbout;
    
    if ( !aboutWin )
        aboutWin= create_aboutWin();

    snprintf( str, 299, _("XTuxCall - %s\n\nCall waiting detector\n\nweb: http://tuxcall.sourceforge.net\ne-mail: inspector__chan@ono.com"), VERSION );
    textAbout= (GtkLabel *)lookup_widget (GTK_WIDGET(aboutWin), "textAbout");
    gtk_label_parse_uline (textAbout, str );
        
    gtk_widget_show( aboutWin );
}


void
hide_about                             (GtkButton       *button,
                                        gpointer         user_data)
{
    gtk_widget_hide( aboutWin );
}



void
on_accept_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
#ifdef DEBUG
    printf("ACCEPT\n");
#endif    
    Action= ACCEPT;
    FuncOnCall= hang;
}


void
on_reject_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
#ifdef DEBUG
    printf("REJECT\n");
#endif    
    Action= REJECT;
    FuncOnCall= NULL;
}

void
on_ask_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
#ifdef DEBUG
    printf("ASK\n");
#endif    
    Action= ASK;
    FuncOnCall= showCallWin;
}

gboolean
on_drawArea_map_event                  (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
    defineGc();
#ifdef DEBUG
    printf("MAP\n");
#endif    
    return FALSE;
}


void
on_callAccept_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
    mySystem( HangCmd );
    gtk_widget_hide( callWin );
    gtk_grab_remove( callWin );
}


void
on_callReject_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
    gtk_widget_hide( callWin );
    gtk_grab_remove( callWin );
}


void
on_config_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_options_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    if ( !confWin )
        confWin= create_confWin();

    gtk_widget_show( confWin );
}


void
on_confOK_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
    char * value;
    int frames;
    float freq;
    char device[FNAMELEN]
       , hangCmd[CMDLEN];
    GtkWidget * f;
    
    f= lookup_widget (GTK_WIDGET(confWin), "confFrames");
    value= gtk_editable_get_chars( GTK_EDITABLE(f), 0, -1 );
    frames= atoi( value );
    if ( frames <= 0 )
    {
        quickMsg( _("\"Frames per update\" must be \nan integer greater than 0") );
        return;
    }
    g_free(value);    

    f= lookup_widget (GTK_WIDGET(confWin), "confDevice");
    value= gtk_editable_get_chars( GTK_EDITABLE(f), 0, -1 );
    strncpy( device, value, FNAMELEN-1 );
    g_free(value);    

    f= lookup_widget (GTK_WIDGET(confWin), "confCommand");
    value= gtk_editable_get_chars( GTK_EDITABLE(f), 0, -1 );
    strncpy( hangCmd, value, CMDLEN-1 );
    g_free(value);    

    f= lookup_widget (GTK_WIDGET(confWin), "confFrequency");
    value= gtk_editable_get_chars( GTK_EDITABLE(f), 0, -1 );
    freq= atof( value );
    if ( frames <= 0 )
    {
        quickMsg( _("\"Frames per update\" must be \nan integer greater than 0") );
        return;
    }

#ifdef DEBUG
    printf( "Device= %s\n", device );
    printf( "Hang cmd= %s\n", hangCmd );
    printf( "frequency= %f\n", freq );
    printf( "frames= %d\n", frames );
#endif

    


    applyConf( device, hangCmd, freq, frames );
    
    gtk_widget_hide( confWin );
}


void
on_confCancel_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
    gtk_widget_hide( confWin );
}


void
on_confWin_map                         (GtkWidget       *widget,
                                        gpointer         user_data)
{
    showConf();
}


void
on_mainWin_map                         (GtkWidget       *widget,
                                        gpointer         user_data)
{
    GtkWidget * w;

    switch( Action )
    {
        case ACCEPT:
#ifdef DEBUG        
            printf( "checked ACCEPT\n" );
#endif            
            w= lookup_widget(GTK_WIDGET(mainWin), "accept");
            gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(w), 1);
            break;
        case REJECT:
#ifdef DEBUG        
            printf( "checked REJECT\n" );
#endif            
            w= lookup_widget(GTK_WIDGET(mainWin), "reject");
            gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(w), 1);
            break;
        case ASK:
#ifdef DEBUG        
            printf( "checked ASK\n" );
#endif            
            w= lookup_widget(GTK_WIDGET(mainWin), "ask");
            gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(w), 1);
            break;
    }

}

